From 06889a9f9c41772ab5259bd7df7d6037d0fc1899 Mon Sep 17 00:00:00 2001
From: Satish MR <satishmr123@gmail.com>
Date: Sat, 10 Nov 2012 12:16:35 +0530
Subject: [PATCH] reach out to armv6 and x86 cpu architectures

This patch tries to propose new way of loading .so files, so that vlc application will
be visible on google play to almost all handsets and run on all of them.

Instead of packaging .so file in .apk file, need to download .so file from VLC's server
as soon as user installs vlc application on his android phone.

By this(removing .so from apk to download) google play will show vlc media player
on all the devices.Once user downloads the apk, based on his CPU architecture,
corresponding .so file can be downloaded and copied to vlc application's private
secured memory area.

This approach will solve current exclusion of almost 180+ armv6 devices & x86 devices.
---
 vlc-android/src/org/videolan/vlc/LibVLC.java       |  44 ++++----
 vlc-android/src/org/videolan/vlc/Util.java         | 116 +++++++++++++++++++++
 .../src/org/videolan/vlc/gui/MainActivity.java     |  35 +++++++
 3 files changed, 174 insertions(+), 21 deletions(-)

diff --git a/vlc-android/src/org/videolan/vlc/LibVLC.java b/vlc-android/src/org/videolan/vlc/LibVLC.java
index 8ad1a49..c171f27 100644
--- a/vlc-android/src/org/videolan/vlc/LibVLC.java
+++ b/vlc-android/src/org/videolan/vlc/LibVLC.java
@@ -22,6 +22,7 @@ package org.videolan.vlc;
 
 import java.util.ArrayList;
 
+import org.videolan.vlc.gui.MainActivity;
 import org.videolan.vlc.gui.video.VideoPlayerActivity;
 
 import android.content.Context;
@@ -63,27 +64,28 @@ public class LibVLC {
 
     /* Load library before object instantiation */
     static {
-        try {
-            if (Build.VERSION.SDK_INT <= 10)
-                System.loadLibrary("iomx-gingerbread");
-            else if (Build.VERSION.SDK_INT <= 13)
-                System.loadLibrary("iomx-hc");
-            else
-                System.loadLibrary("iomx-ics");
-        } catch (Throwable t) {
-            Log.w(TAG, "Unable to load the iomx library: " + t);
-        }
-        try {
-            System.loadLibrary("vlcjni");
-        } catch (UnsatisfiedLinkError ule) {
-            Log.e(TAG, "Can't load vlcjni library: " + ule);
-            /// FIXME Alert user
-            System.exit(1);
-        } catch (SecurityException se) {
-            Log.e(TAG, "Encountered a security issue when loading vlcjni library: " + se);
-            /// FIXME Alert user
-            System.exit(1);
-        }
+    	if (MainActivity.iOmxNativeLibPath != null) 
+    	{
+	        try {
+	               System.load(MainActivity.iOmxNativeLibPath);
+	        } catch (Throwable t) {
+	            Log.w(TAG, "Unable to load the iomx library: " + t);
+	        }
+    	}
+    	if (MainActivity.ilibVlcJniNativeLibPath != null)
+    	{
+	        try {
+	            System.load(MainActivity.ilibVlcJniNativeLibPath);
+	        } catch (UnsatisfiedLinkError ule) {
+	            Log.e(TAG, "Can't load vlcjni library: " + ule);
+	            /// FIXME Alert user
+	            System.exit(1);
+	        } catch (SecurityException se) {
+	            Log.e(TAG, "Encountered a security issue when loading vlcjni library: " + se);
+	            /// FIXME Alert user
+	            System.exit(1);
+	        }
+    	}
     }
 
     /**
diff --git a/vlc-android/src/org/videolan/vlc/Util.java b/vlc-android/src/org/videolan/vlc/Util.java
index 601a58a..cf20b1c 100644
--- a/vlc-android/src/org/videolan/vlc/Util.java
+++ b/vlc-android/src/org/videolan/vlc/Util.java
@@ -23,6 +23,9 @@ package org.videolan.vlc;
 import java.io.BufferedReader;
 import java.io.ByteArrayInputStream;
 import java.io.File;
+import java.io.FileInputStream;
+import java.io.FileNotFoundException;
+import java.io.FileOutputStream;
 import java.io.FileReader;
 import java.io.IOException;
 import java.io.InputStream;
@@ -33,12 +36,17 @@ import java.util.HashSet;
 import java.util.Locale;
 import java.util.Properties;
 
+import org.videolan.vlc.gui.audio.AudioUtil;
+
 import android.content.Context;
+import android.content.pm.PackageManager;
+import android.content.pm.PackageManager.NameNotFoundException;
 import android.graphics.Bitmap;
 import android.net.Uri;
 import android.os.Environment;
 import android.telephony.TelephonyManager;
 import android.util.DisplayMetrics;
+import android.util.Log;
 import android.util.TypedValue;
 import android.view.Display;
 import android.view.View;
@@ -380,4 +388,112 @@ public class Util {
             return true;
         }
     }
+    
+	public static final String ARMV6 = "armv6";
+	public static final String ARMV7 = "armv7";
+	public static final String X86 = "x86";
+	public static final String NONE = "NONE";
+	public static boolean hasFPUFlag = false;
+	public static String CPUArch = null;
+	/**
+	 * Returns the CPU architecture.
+	 */
+    public static String getCPUArchType () {
+        try {
+            FileReader fileReader = new FileReader("/proc/cpuinfo");
+            BufferedReader br = new BufferedReader(fileReader);
+            String line;
+            while((line = br.readLine()) != null) {
+                if(line.contains("ARMv7")) {
+                    CPUArch = ARMV7;
+                    break;
+                }
+                if(line.contains("ARMv6")) {
+                    CPUArch = ARMV6;
+                    break;
+                }
+                if(line.contains("clflush size")) {
+                    CPUArch = X86;
+                    break;
+                }
+                if((hasFPUFlag == false) && line.contains("vfp"))
+                	hasFPUFlag = true;
+//                Other architectures
+//                if ()
+//                {
+//                }
+             }
+            fileReader.close();
+        } catch(IOException ex){
+            ex.printStackTrace();
+            return null;
+        }
+    	return CPUArch;
+    }
+    
+    public static String getCPUArch () {
+    	return CPUArch;
+    }
+    
+    public static boolean getFPU () {
+    	return hasFPUFlag;
+    }
+    
+	/**
+	 * Copies the given file to application's persistent memory (like '/data/data/package-name/solibs' folder).
+	 * Returns the path to copied destination file. Null is returned if copy gets failed.
+	 */
+	public static String copyNativeFile(File file) {
+		String NativeLibraryDir = null;
+		PackageManager pm = VLCApplication.getAppContext().getPackageManager();
+		try {
+//			if (isGingerbreadOrLater()) {
+//				NativeLibraryDir = pm.getApplicationInfo(VLCApplication.getAppContext().getPackageName(), PackageManager.GET_SHARED_LIBRARY_FILES).nativeLibraryDir;
+//			} else {
+				NativeLibraryDir = pm.getApplicationInfo(VLCApplication.getAppContext().getPackageName(), PackageManager.GET_SHARED_LIBRARY_FILES).dataDir;
+				NativeLibraryDir = NativeLibraryDir + "/solibs/";
+//			}
+		} catch (NameNotFoundException e1) {
+			e1.printStackTrace();
+			return null;
+		}
+		File targetFolder = new File(NativeLibraryDir);
+		if(!targetFolder.exists()) {
+			targetFolder.mkdirs();
+		}
+		String soFilePath = targetFolder.getAbsolutePath() + "/" + file.getName();
+		
+		try {
+			FileOutputStream oStream = new FileOutputStream(soFilePath);
+			FileInputStream iStream = new FileInputStream(file);
+			Log.d(TAG, "library copying from " + file.toString() + " to " + soFilePath);
+			byte[] buffer = new byte[512];
+			int length = 0;
+			while ((length = iStream.read(buffer)) > 0) {
+				oStream.write(buffer, 0 , length);
+			}
+			oStream.close();
+			iStream.close();
+		} catch (FileNotFoundException e) {
+			e.printStackTrace();
+			return null;
+		} catch (IOException e) {
+			e.printStackTrace();
+			return null;
+		}
+		return soFilePath;
+	}
+    
+    /* This method has to be implemented */
+    public static String Download_NativeSO_to_CacheDirectory(String CPUArch, boolean hasFPUFlag, String SOname) {
+// Download the stable SOname file for CPUArch architecture from VLC's server and copy it to AudioUtil.CACHE_DIR directory
+/* CPUArch -- hasFPUFlag  -- .So to be Downloaded
+   ARMV7   --  NA         -- Libvlcjni_ARMV7.so 
+   ARMV6   --  TRUE       -- Libvlcjni_ARMV6_hasFPU.so 
+   ARMV6   --  FALSE      -- Libvlcjni_ARMV6_noFPU.so 
+   X86     --  NA         -- Libvlcjni_X86.so 
+*/
+    	String sofilepath_cachedir = AudioUtil.CACHE_DIR + "/" + SOname;//download destination
+    	return sofilepath_cachedir;
+    }
 }
diff --git a/vlc-android/src/org/videolan/vlc/gui/MainActivity.java b/vlc-android/src/org/videolan/vlc/gui/MainActivity.java
index a963e37..11667fe 100644
--- a/vlc-android/src/org/videolan/vlc/gui/MainActivity.java
+++ b/vlc-android/src/org/videolan/vlc/gui/MainActivity.java
@@ -20,6 +20,7 @@
 
 package org.videolan.vlc.gui;
 
+import java.io.File;
 import java.util.ArrayList;
 
 import org.videolan.vlc.AudioService;
@@ -32,6 +33,7 @@ import org.videolan.vlc.ThumbnailerManager;
 import org.videolan.vlc.Util;
 import org.videolan.vlc.VLCCallbackTask;
 import org.videolan.vlc.gui.SidebarAdapter.SidebarEntry;
+import org.videolan.vlc.gui.audio.AudioUtil;
 import org.videolan.vlc.gui.video.VideoListAdapter;
 import org.videolan.vlc.gui.video.VideoGridFragment;
 import org.videolan.vlc.interfaces.ISortable;
@@ -49,6 +51,7 @@ import android.content.SharedPreferences.Editor;
 import android.content.pm.PackageInfo;
 import android.content.pm.PackageManager.NameNotFoundException;
 import android.content.res.Configuration;
+import android.os.Build;
 import android.os.Bundle;
 import android.os.Handler;
 import android.preference.PreferenceManager;
@@ -80,6 +83,8 @@ import com.slidingmenu.lib.SlidingMenu;
 
 public class MainActivity extends SherlockFragmentActivity {
     public final static String TAG = "VLC/MainActivity";
+    public static String iOmxNativeLibPath = null;
+    public static String ilibVlcJniNativeLibPath = null;
 
     protected static final String ACTION_SHOW_PROGRESSBAR = "org.videolan.vlc.gui.ShowProgressBar";
     protected static final String ACTION_HIDE_PROGRESSBAR = "org.videolan.vlc.gui.HideProgressBar";
@@ -110,6 +115,36 @@ public class MainActivity extends SherlockFragmentActivity {
 
     @Override
     protected void onCreate(Bundle savedInstanceState) {
+    	String CPUArch = Util.getCPUArchType();
+    	boolean hasFPU = Util.getFPU();
+    	String iOmxLibraryName = null;
+    	if (CPUArch != null) {
+    		AudioUtil.prepareCacheFolder(getApplicationContext());
+    		//Download the .so files from VLC server corresponding to detected CPU architecture
+    		String sofile_cachedir = Util.Download_NativeSO_to_CacheDirectory(CPUArch, hasFPU, "libvlcjni.so");
+    		
+    		//Store .so files in CACHE_DIR 
+            if (Build.VERSION.SDK_INT <= android.os.Build.VERSION_CODES.GINGERBREAD)
+            	iOmxLibraryName = "libiomx-gingerbread.so";
+            else if (Build.VERSION.SDK_INT <= android.os.Build.VERSION_CODES.HONEYCOMB)
+            	iOmxLibraryName = "libiomx-hc.so";
+            else
+            	iOmxLibraryName = "libiomx-ics.so";
+            
+            String iomxsofile_cachedir = Util.Download_NativeSO_to_CacheDirectory(CPUArch, hasFPU, iOmxLibraryName);
+            //Before proceeding, synchronous download of native libraries must be completed
+            //Copy Native library from cache directory to application's directory.
+    		iOmxNativeLibPath = Util.copyNativeFile(new File(iomxsofile_cachedir));
+    		if (iOmxNativeLibPath == null) {
+    			Log.e(TAG, "Sorry copying the native library failed");
+    		}
+    		ilibVlcJniNativeLibPath = Util.copyNativeFile(new File(sofile_cachedir));
+    		if (ilibVlcJniNativeLibPath == null) {
+    			Log.e(TAG, "Sorry copying the native library failed");
+    			System.exit(1);
+    		}
+    	}
+    	
         if (!Util.hasCompatibleCPU()) {
             Log.e(TAG, Util.getErrorMsg());
             Intent i = new Intent(this, CompatErrorActivity.class);
-- 
1.8.0

